@@ -33,6 +33,9 @@
 #include "bakery/View/View.h"
 #include <bakery/Utilities/BusyCursor.h>
 
+namespace Glom
+{
+
 class LayoutItem_GroupBy;
 class LayoutItem_Summary;
 class LayoutItem_VerticalGroup;
@@ -86,22 +89,23 @@ public:
   static type_vecFields get_fields_for_table_from_database(const Glib::ustring& table_name, bool including_system_fields = false);
   static bool get_field_exists_in_database(const Glib::ustring& table_name, const Glib::ustring& field_name);
 
-  /** Create an appropriate title for an ID string.
-   * For instance, date_of_birth would become Date Of Birth.
-   */
-  static Glib::ustring util_title_from_string(const Glib::ustring& text);
 
   //This is const because const means not changing this instance, not whether we change the database.
-  virtual Glib::RefPtr<Gnome::Gda::DataModel> Query_execute(const Glib::ustring& strQuery, Gtk::Window* parent_window = 0) const;
+  static Glib::RefPtr<Gnome::Gda::DataModel> query_execute(const Glib::ustring& strQuery, Gtk::Window* parent_window = 0);
 
   bool add_standard_groups();
   bool add_standard_tables() const;
 
   bool create_table(const sharedptr<const TableInfo>& table_info, const Document_Glom::type_vecFields& fields) const;
   bool create_table_add_missing_fields(const sharedptr<const TableInfo>& table_info, const Document_Glom::type_vecFields& fields) const;
+
+  /// Also saves the table information in the document:
+  bool create_table_with_default_fields(const Glib::ustring& table_name);
+
   bool insert_example_data(const Glib::ustring& table_name) const;
 
   typedef std::vector< sharedptr<LayoutItem_Field> > type_vecLayoutFields;
+  typedef std::vector< sharedptr<const LayoutItem_Field> > type_vecConstLayoutFields;
 
 protected:
   sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
@@ -120,6 +124,7 @@ protected:
   type_vecStrings get_table_names_from_database(bool ignore_system_tables = false) const;
 
   bool get_table_exists_in_database(const Glib::ustring& table_name) const;
+  bool get_relationship_exists(const Glib::ustring& table_name, const Glib::ustring& relationship_name);
 
   type_vecFields get_fields_for_table(const Glib::ustring& table_name, bool including_system_fields = false) const;
   sharedptr<Field> get_fields_for_table_one_field(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
@@ -188,37 +193,67 @@ protected:
     Gnome::Gda::Value m_key_value;
   };
 
+
+  class LayoutFieldInRecord
+  {
+  public:
+    LayoutFieldInRecord()
+    {}
+
+    LayoutFieldInRecord(const sharedptr<const LayoutItem_Field>& layout_item, const Glib::ustring& parent_table_name, const sharedptr<const Field>& parent_key, const Gnome::Gda::Value& key_value)
+    : m_key_value(key_value)
+    {
+      m_field = layout_item;
+      m_table_name = parent_table_name;
+      m_key = parent_key;
+    }
+
+    FieldInRecord get_fieldinrecord(const Document_Glom& document) const
+    {
+      return FieldInRecord(m_field, m_table_name, m_key, m_key_value, document);
+    }
+
+    //Identify the field:
+    Glib::ustring m_table_name;
+    sharedptr<const LayoutItem_Field> m_field;
+
+    //Identify the record:
+    sharedptr<const Field> m_key;
+    Gnome::Gda::Value m_key_value;
+  };
+
   /** Calculate values for fields, set them in the database, and show them in the layout.
    * @param field_changed The field that has changed, causing other fields to be recalculated because they use its value.
    * @param primary_key The primary key field for this table.
    * @param priamry_key_value: The primary key value for this record.
    * @param first_calc_field: false if this is called recursively.
    */
-  void do_calculations(const FieldInRecord& field_changed, bool first_calc_field);
+  void do_calculations(const LayoutFieldInRecord& field_changed, bool first_calc_field);
 
   typedef std::map<Glib::ustring, CalcInProgress> type_field_calcs;
 
+  typedef std::list< sharedptr<LayoutItem_Field> > type_list_field_items;
+  typedef std::list< sharedptr<const LayoutItem_Field> > type_list_const_field_items;
+
   /** Get the fields whose values should be recalculated when @a field_name changes.
    */
-  type_field_calcs get_calculated_fields(const Glib::ustring& table_name, const Glib::ustring& field_name);
-
-  typedef std::list< sharedptr<LayoutItem_Field> > type_list_field_items;
+  type_list_const_field_items get_calculated_fields(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field);
 
   /** Get the fields used, if any, in the calculation of this field.
    */
-  type_list_field_items get_calculation_fields(const Glib::ustring& table_name, const sharedptr<const Field>& field);
+  type_list_const_field_items get_calculation_fields(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field);
 
-  void calculate_field(const FieldInRecord& field_in_record);
+  void calculate_field(const LayoutFieldInRecord& field_in_record);
 
   void calculate_field_in_all_records(const Glib::ustring& table_name, const sharedptr<const Field>& field);
   void calculate_field_in_all_records(const Glib::ustring& table_name, const sharedptr<const Field>& field, const sharedptr<const Field>& primary_key);
 
   typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_fields;
   //TODO: Performance: This is massively inefficient:
-  type_map_fields get_record_field_values(const Glib::ustring& table_name, const sharedptr<const Field> primary_key, const Gnome::Gda::Value& primary_key_value);
+  type_map_fields get_record_field_values_for_calculation(const Glib::ustring& table_name, const sharedptr<const Field> primary_key, const Gnome::Gda::Value& primary_key_value);
 
 
-  void do_lookups(const FieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+  void do_lookups(const LayoutFieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
 
   typedef std::pair< sharedptr<LayoutItem_Field>, sharedptr<Relationship> > type_pairFieldTrigger;
   typedef std::list<type_pairFieldTrigger> type_list_lookups;
@@ -234,44 +269,20 @@ protected:
   Gnome::Gda::Value get_lookup_value(const Glib::ustring& table_name, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value & key_value);
 
 
-  virtual void refresh_related_fields(const FieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+  virtual void refresh_related_fields(const LayoutFieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
 
 
 
-  bool set_field_value_in_database(const FieldInRecord& field_in_record, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
-  bool set_field_value_in_database(const FieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
+  bool set_field_value_in_database(const LayoutFieldInRecord& field_in_record, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
+  bool set_field_value_in_database(const LayoutFieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value, bool use_current_calculations = false, Gtk::Window* parent_window = 0);
 
+  //Get a single field value from the database.
+  Gnome::Gda::Value get_field_value_in_database(const LayoutFieldInRecord& field_in_record, Gtk::Window* parent_window);
 
-  type_vecStrings get_database_groups() const;
-  type_vecStrings get_database_users(const Glib::ustring& group_name = Glib::ustring()) const;
-  Privileges get_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name) const;
-  void set_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name, const Privileges& privs, bool developer_privs = false);
-  Glib::ustring get_user_visible_group_name(const Glib::ustring& group_name) const;
-
-  type_vecStrings get_groups_of_user(const Glib::ustring& user) const;
-  bool get_user_is_in_group(const Glib::ustring& user, const Glib::ustring& group) const;
-  Privileges get_current_privs(const Glib::ustring& table_name) const;
 
   SystemPrefs get_database_preferences() const;
   void set_database_preferences(const SystemPrefs& prefs);
 
-  void report_build(const FoundSet& found_set, const sharedptr<const Report>& report, Gtk::Window* parent_window = 0);
-
-  void report_build_groupby(const FoundSet& found_set_parent, xmlpp::Element& parent_node, const sharedptr<LayoutItem_GroupBy>& group_by);
-  void report_build_groupby_children(const FoundSet& found_set, xmlpp::Element& nodeGroupBy, const sharedptr<LayoutItem_GroupBy>& group_by);
-  void report_build_summary(const FoundSet& found_set_parent, xmlpp::Element& parent_node, const sharedptr<LayoutItem_Summary>& summary);
-  void report_build_headerfooter(const FoundSet& found_set, xmlpp::Element& parent_node, const sharedptr<LayoutGroup>& group);
-
-  typedef std::vector< sharedptr<LayoutItem> > type_vecLayoutItems;
-
-  void report_build_records(const FoundSet& found_set, xmlpp::Element& parent_node, const type_vecLayoutItems& items, bool one_record_only = false);
-  void report_build_records_get_fields(const FoundSet& found_set, const sharedptr<LayoutGroup>& group, type_vecLayoutFields& items);
-  void report_build_records_field(const FoundSet& found_set, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Field>& field, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& colField, bool vertical = false);
-  void report_build_records_text(const FoundSet& found_set, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Text>& textobject, bool vertical = false);
-  void report_build_records_image(const FoundSet& found_set, xmlpp::Element& nodeParent, const sharedptr<const LayoutItem_Image>& imageobject, bool vertical = false);
-  void report_build_records_vertical_group(const FoundSet& found_set, xmlpp::Element& vertical_group_node, const sharedptr<LayoutItem_VerticalGroup>& group, const Glib::RefPtr<Gnome::Gda::DataModel>& datamodel, guint row, guint& field_index);
-
-
   Gnome::Gda::Value auto_increment_insert_first_if_necessary(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
   /** Get the next auto-increment value for this primary key, from the glom system table.
@@ -285,41 +296,43 @@ protected:
    */
   void recalculate_next_auto_increment_value(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
+  bool get_field_value_is_unique(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
+
+  bool check_entered_value_for_uniqueness(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value, Gtk::Window* parent_window);
+  bool check_entered_value_for_uniqueness(const Glib::ustring& table_name, const Gtk::TreeModel::iterator& /* row */,  const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value, Gtk::Window* parent_window);
+
   virtual bool fill_from_database();
   virtual void fill_end(); //Call this from the end of fill_from_database() overrides.
 
   virtual void on_userlevel_changed(AppState::userlevels userlevel);
 
   type_vecLayoutFields get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document_Glom::type_mapLayoutGroupSequence& mapGroupSequence) const;
-  void get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const sharedptr<const LayoutGroup>& group, type_vecLayoutFields& vecFields) const;
+  void get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vecFields& all_db_fields, const sharedptr<LayoutGroup>& group, type_vecLayoutFields& vecFields) const;
+
+  /** Get the relationship into which the row button should navigate,
+   * or the relationship itself, if the navigation_main output parameter is set to true after calling this method.
+   * (If that should be chosen automatically, by looking at the fields in the portal.)
+   */
+  sharedptr<const UsesRelationship> get_portal_navigation_relationship_automatic(const sharedptr<LayoutItem_Portal>& portal, bool& navigation_main) const;
+  sharedptr<const LayoutItem_Field> get_field_is_from_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal) const;
+  sharedptr<const LayoutItem_Field> get_field_identifies_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal, sharedptr<const Relationship>& used_in_relationship) const;
+
+  bool get_primary_key_is_in_foundset(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value);
 
   static bool get_field_primary_key_index_for_fields(const type_vecFields& fields, guint& field_column);
   static bool get_field_primary_key_index_for_fields(const type_vecLayoutFields& fields, guint& field_column);
 
-  static Glib::ustring util_string_from_decimal(guint decimal);
-  static guint util_decimal_from_string(const Glib::ustring& str);
 
-  static bool util_string_has_whitespace(const Glib::ustring& text);
 
   static type_vecStrings util_vecStrings_from_Fields(const type_vecFields& fields);
 
-  //Utlility functions to help with the odd formats of postgres internal catalog fields:
-  static Glib::ustring string_trim(const Glib::ustring& str, const Glib::ustring& to_remove);
-  static type_vecStrings string_separate(const Glib::ustring& str, const Glib::ustring& separator);
-  static type_vecStrings pg_list_separate(const Glib::ustring& str);
 
-  /** @param not_extras If this is true, then do not set extra details, such as NOT NULL. You should do that later, when you are ready.
-   */
-  bool postgres_add_column(const Glib::ustring& table_name, const sharedptr<const Field>& field, bool not_extras = false) const;
-
-  /** @param set_anyway If this is true, then set the extra details even if @field_old has the same properties.
-   */
-  void postgres_change_column_extras(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, bool set_anyway = false) const;
-
-  void handle_error(const std::exception& ex) const; //TODO_port: This is probably useless now.
-  bool handle_error() const;
+  static void handle_error(const std::exception& ex); //TODO_port: This is probably useless now.
+  static bool handle_error();
 
   type_field_calcs m_FieldsCalculationInProgress; //Prevent circular calculations and recalculations.
 };
 
+} //namespace Glom
+
 #endif //BASE_DB_H
