@@ -23,7 +23,7 @@
 
 #include <gtkmm/frame.h>
 #include "bakery/View/View_Composite.h"
-#include "document/document_glom.h"
+#include <glom/libglom/document/document_glom.h>
 
 #include "dialog_glom.h"
 
@@ -36,11 +36,15 @@
 #include "mode_design/dialog_fields.h"
 #include "mode_design/dialog_relationships.h"
 #include "dialog_connection.h"
-#include "utils.h"
+#include <glom/libglom/utils.h>
 
 #include "mode_data/box_data_list_related.h" //only for m_HackToFixLinkerError.
 
+namespace Glom
+{
+
 class Dialog_Layout_Report;
+class Dialog_AddRelatedTable;
 
 class Frame_Glom :
   public PlaceHolder,
@@ -70,23 +74,29 @@ public:
   //virtual void on_menu_Navigate_Database();
   //virtual void do_menu_Navigate_Database(bool bUseList = true);
   void on_menu_Tables_EditTables();
+  void on_menu_Tables_AddRelatedTable();
   void do_menu_Navigate_Table(bool open_default = false);
 
   void on_menu_Tables_EditReports();
 
-  virtual void on_menu_developer_database_preferences();
-  virtual void on_menu_developer_fields();
-  virtual void on_menu_developer_relationships_overview();
-  virtual void on_menu_developer_relationships();
-  virtual void on_menu_developer_users();
-  virtual void on_menu_developer_layout();
-  virtual void on_menu_developer_reports();
+  void on_menu_developer_database_preferences();
+
+  void on_menu_developer_fields();
+  void do_menu_developer_fields(Gtk::Window& parent);
+
+  void on_menu_developer_relationships_overview();
+  void on_menu_developer_relationships();
+  void on_menu_developer_users();
+  void on_menu_developer_layout();
+  void on_menu_developer_reports();
 
   void on_developer_dialog_hide();
   void on_dialog_layout_report_hide();
   void on_dialog_reports_hide();
   void on_dialog_tables_hide();
 
+  void on_dialog_add_related_table_request_edit_fields();
+
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
 
@@ -107,13 +117,13 @@ public:
   bool connection_request_password_and_attempt();
 
   //Show the dialog to request the password, and choose an unused database name.
-  bool Frame_Glom::connection_request_password_and_choose_new_database_name();
+  bool connection_request_password_and_choose_new_database_name();
 
   ///Create the database for new documents, showing the Connection dialog
   bool create_database(const Glib::ustring& database_name, const Glib::ustring& title, bool request_password = true);
 
-  void export_data_to_string(Glib::ustring& the_string, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause = Glib::ustring());
-  void export_data_to_stream(std::ostream& the_stream, const Glib::ustring table_name, const Document_Glom::type_mapLayoutGroupSequence& sequence, const Glib::ustring& where_clause = Glib::ustring());
+  void export_data_to_string(Glib::ustring& the_string, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence);
+  void export_data_to_stream(std::ostream& the_stream, const FoundSet& found_set, const Document_Glom::type_mapLayoutGroupSequence& sequence);
 
   /** Show the table again. For instance, if the document has changed, or we want to display it differently.
    */
@@ -123,7 +133,7 @@ protected:
 
   //virtual void set_document(Document_Glom* pDocument); //override
 
-  void show_table(const Glib::ustring& strTableName);
+  void show_table(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value_for_details = Gnome::Gda::Value());
   void show_table_title();
   void update_table_in_document_from_database();
 
@@ -138,14 +148,16 @@ protected:
   void alert_no_table();
 
   //Signal handlers:
-  void on_notebook_find_criteria(const Glib::ustring& strWhereClause);
+  void on_notebook_find_criteria(const Glib::ustring& where_clause);
   void on_button_quickfind();
   void on_button_find_all();
   void on_notebook_data_record_details_requested(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
   void on_userlevel_changed(AppState::userlevels userlevel);
 
+  void on_dialog_add_related_table_response(int response);
+
   //Member data:
-  Glib::ustring m_strTableName;
+  Glib::ustring m_table_name;
 
   //Child widgets:
   Gtk::Label* m_pLabel_Name;
@@ -177,6 +189,7 @@ protected:
   //Developer:
   Dialog_Fields* m_pDialog_Fields;
   Dialog_Relationships* m_pDialog_Relationships;
+  Dialog_AddRelatedTable* m_dialog_addrelatedtable;
 
   Dialog_Connection* m_pDialogConnection;
   Gtk::Dialog* m_pDialogConnectionFailed;
@@ -186,4 +199,6 @@ protected:
   Box_Data_List_Related m_HackToFixLinkerError; //The implementation of this class does not seem to be in the library unless I do this. murrayc.
 };
 
+} //namespace Glom
+
 #endif //FRAME_GLOM_H
