@@ -21,6 +21,9 @@
 #include "box_data_details_find.h"
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_Data_Details_Find::Box_Data_Details_Find()
 : Box_Data_Details(false)
 {
@@ -40,12 +43,14 @@ Box_Data_Details_Find::~Box_Data_Details_Find()
 
 bool Box_Data_Details_Find::init_db_details(const Glib::ustring& table_name)
 {
-  return Box_Data_Details::init_db_details(table_name, Gnome::Gda::Value());
+  FoundSet found_set;
+  found_set.m_table_name = table_name;
+  return Box_Data_Details::init_db_details(found_set, Gnome::Gda::Value());
 }
 
 bool Box_Data_Details_Find::fill_from_database()
 {
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor busy_cursor(get_app_window());
 
   bool result = Box_DB_Table::fill_from_database();
 
@@ -62,7 +67,7 @@ void Box_Data_Details_Find::fill_related()
   //m_Notebook_Related.pages().clear();
 
   //Get relationships from the document:
-  Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_strTableName);
+  Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_table_name);
 
   //Add the relationships:
   for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
@@ -72,23 +77,23 @@ void Box_Data_Details_Find::fill_related()
 
 
      Box_Data_List_Related* pBox = Gtk::manage(new Box_Data_List_Related());
-     std::cout <<  "Box_Data_Details::fill_related() 2:" << relationship.get_name() << std::endl;
-     m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship.get_name()) );
+     std::cout <<  "Box_Data_Details::fill_related() 2:" << relationship->get_name() << std::endl;
+     m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship->get_name()) );
      std::cout <<  "Box_Data_Details::fill_related() 2.5:" << std::endl;
 
-     guint rowKey = m_FieldsShown.get_index(relationship.get_from_field());
+     guint rowKey = m_FieldsShown.get_index(relationship->get_from_field());
      Glib::ustring strKeyValue = m_AddDel.get_value(rowKey);
      strKeyValue = m_FieldsShown[rowKey].sql(strKeyValue); //Quote/Escape it if necessary.
 
      std::cout <<  "Box_Data_Details::fill_related() 3:" << std::endl;
-     pBox->init_db_details(get_database_name(), relationship.get_to_table(), relationship.get_to_field(), strKeyValue);
+     pBox->init_db_details(get_database_name(), relationship->get_to_table(), relationship->get_to_field(), strKeyValue);
      pBox->show_all();
      */
   }
 }
 
 
-void Box_Data_Details_Find::on_flowtable_field_edited(const LayoutItem_Field& /* id */, const Gnome::Gda::Value& /* value */)
+void Box_Data_Details_Find::on_flowtable_field_edited(const sharedptr<const LayoutItem_Field>& /* id */, const Gnome::Gda::Value& /* value */)
 {
   //Don't do anything.
   //This just blocks the method in the base class.
@@ -99,3 +104,4 @@ Gtk::Widget* Box_Data_Details_Find::get_default_button() //override
   return &m_Button_Find;
 }
  
+} //namespace Glom
