@@ -21,6 +21,9 @@
 #include "box_data_list_find.h"
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 Box_Data_List_Find::Box_Data_List_Find()
 : m_HBox(false, 6)
 {
@@ -45,7 +48,7 @@ void Box_Data_List_Find::create_layout()
 
 bool Box_Data_List_Find::fill_from_database()
 {
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor busy_cursor(get_app_window());
 
   bool result = Box_DB_Table::fill_from_database();
 
@@ -81,3 +84,12 @@ void Box_Data_List_Find::on_adddel_user_added(const Gtk::TreeModel::iterator& /*
 {
   //Ignore this, instead of updating the database, as the base class does.
 }
+
+bool Box_Data_List_Find::init_db_details(const Glib::ustring& table_name)
+{
+  FoundSet found_set;
+  found_set.m_table_name = table_name;
+  return Box_Data_List::init_db_details(found_set);
+}
+
+} //namespace Glom
