@@ -31,7 +31,8 @@
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <glibmm/i18n.h>
 
-
+namespace Glom
+{
 
 FlowTableWithFields::Info::Info()
 : m_first(0),
@@ -452,7 +453,7 @@ void FlowTableWithFields::add_field_at_position(const sharedptr<LayoutItem_Field
 }
 
 
-void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Button>& layoutitem_button, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Button>& layoutitem_button, const Glib::ustring& /* table_name */, const type_list_layoutwidgets::iterator& add_before)
 {
   //Add the widget:
   ButtonGlom* button = Gtk::manage(new ButtonGlom());
@@ -471,7 +472,7 @@ void FlowTableWithFields::add_button_at_position(const sharedptr<LayoutItem_Butt
   add(*button);
 }
 
-void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_Text>& layoutitem_text, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_Text>& layoutitem_text, const Glib::ustring& /* table_name */, const type_list_layoutwidgets::iterator& add_before)
 {
   //Add the widget:
   Gtk::Alignment* alignment_label = Gtk::manage(new Gtk::Alignment());
@@ -502,7 +503,7 @@ void FlowTableWithFields::add_textobject_at_position(const sharedptr<LayoutItem_
   }
 }
 
-void FlowTableWithFields::add_imageobject_at_position(const sharedptr<LayoutItem_Image>& layoutitem_image, const Glib::ustring& table_name, const type_list_layoutwidgets::iterator& add_before)
+void FlowTableWithFields::add_imageobject_at_position(const sharedptr<LayoutItem_Image>& layoutitem_image, const Glib::ustring& /* table_name */, const type_list_layoutwidgets::iterator& add_before)
 {
   //Add the widget:
   ImageGlom* image = Gtk::manage(new ImageGlom());
@@ -1038,7 +1039,7 @@ void FlowTableWithFields::on_portal_user_requested_details(Gnome::Gda::Value pri
     Gnome::Gda::Value doubly_related_primary_key;
     portal_box->get_suitable_record_to_view_details(primary_key_value, doubly_related_table_name, doubly_related_primary_key);
 
-    if(!(doubly_related_table_name.empty()))
+    if(!(doubly_related_table_name.empty()) && !Conversions::value_is_empty(doubly_related_primary_key))
        signal_requested_related_details().emit(doubly_related_table_name, doubly_related_primary_key);
   }
   else
@@ -1053,3 +1054,4 @@ void FlowTableWithFields::on_flowtable_requested_related_details(const Glib::ust
   signal_requested_related_details().emit(table_name, primary_key_value);
 }
 
+} //namespace Glom
